# Support functions for ddd
#
# A generic class to build line-oriented command interpreters
# Slightly modified version of Python library module cmd.py

import string
import sys
import linecache

PROMPT = '(Cmd) '
IDENTCHARS = string.letters + string.digits + '_'

class Cmd:

    def __init__(self):
        self.prompt = PROMPT
        self.identchars = IDENTCHARS
        self.lastcmd = ''

    def cmdloop(self):
        stop = None
        while not stop:
            if self.cmdqueue:
                line = self.cmdqueue[0]
                del self.cmdqueue[0]
            else:
                try:
                    line = raw_input(self.prompt)
                except EOFError:
                    # ignore EOF errors ... use quit to end loop
                    continue
            stop = self.onecmd(line)
            stop = stop or self.alldie

    def onecmd(self, line):
        line = string.strip(line)
        if not line:
            line = self.lastcmd
        else:
            self.lastcmd = line
        if not line:
            return 0
        # Ignore comments
        if line[0] == '#':
            return 0
        i, n = 0, len(line)
        while i < n and line[i] in self.identchars: i = i+1
        cmd, arg = line[:i], string.strip(line[i:])
        if cmd == '':
            return self.default(line)
        else:
            try:
                func = getattr(self, 'do_' + cmd)
            except AttributeError:
                return self.default(line)
            return func(arg)

    def default(self, line):
        print '*** Unknown syntax:', line

    def do_help(self, arg):
        if arg:
            try:
                func = getattr(self, 'help_' + arg)
            except:
                print 'Undefined command: "%s".  Try "help".' % (arg,)
                return
            func()
        else:
            ##import newdir
            names = newdir_dir(self.__class__)
            cmds_doc = []
            cmds_undoc = []
            help = {}
            for name in names:
                if name[:5] == 'help_':
                    help[name[5:]]=1
            for name in names:
                if name[:3] == 'do_':
                    cmd=name[3:]
                    if help.has_key(cmd):
                        cmds_doc.append(cmd)
                        del help[cmd]
                    else:
                        cmds_undoc.append(cmd)
            print 
            print "Type 'quit' to exit debugger"
            self.print_topics("Documented commands (type help " \
                                  "<topic>):",cmds_doc, 15, 80)
            self.print_topics("Miscellaneous help topics:",
                                  help.keys(), 15, 80)
            self.print_topics("Undocumented commands:", 
                                  cmds_undoc, 15, 80)

    def print_topics(self, header, cmds, cmdlen, maxcol):
        if cmds:
            print header;
            print "="*len(header)
            (cmds_per_line,junk)=divmod(maxcol,cmdlen)
            col=cmds_per_line
            for cmd in cmds:
                if col==0: print
                print (("%-"+`cmdlen`+"s") % cmd),
                col = (col+1) % cmds_per_line
            print "\n"


#
# This is copied from lib-old/newdir.py, which has now been removed from Python:
#

# New dir() function
# This should be the new dir(), except that it should still list
# the current local name space by default

def newdir_listattrs(x):
	try:
		dictkeys = x.__dict__.keys()
	except (AttributeError, TypeError):
		dictkeys = []
	#
	try:
		methods = x.__methods__
	except (AttributeError, TypeError):
		methods = []
	#
	try:
		members = x.__members__
	except (AttributeError, TypeError):
		members = []
	#
	try:
		the_class = x.__class__
	except (AttributeError, TypeError):
		the_class = None
	#
	try:
		bases = x.__bases__
	except (AttributeError, TypeError):
		bases = ()
	#
	total = dictkeys + methods + members
	if the_class:
		# It's a class instace; add the class's attributes
		# that are functions (methods)...
		class_attrs = newdir_listattrs(the_class)
		class_methods = []
		for name in class_attrs:
			if is_function(getattr(the_class, name)):
				class_methods.append(name)
		total = total + class_methods
	elif bases:
		# It's a derived class; add the base class attributes
		for base in bases:
			base_attrs = newdir_listattrs(base)
			total = total + base_attrs
	total.sort()
	return total
	i = 0
	while i+1 < len(total):
		if total[i] == total[i+1]:
			del total[i+1]
		else:
			i = i+1
	return total


# Helper to recognize functions

def newdir_is_function(x):
	return type(x) == type(is_function)


# Approximation of builtin dir(); but note that this lists the user's
# variables by default, not the current local name space.

def newdir_dir(x = None):
	if x is not None:
		return newdir_listattrs(x)
	else:
		import __main__
		return newdir_listattrs(__main__)
